from __future__ import division, absolute_import, print_function

try:
    import enum
    SUPPORTS_ENUM = True
except ImportError:
    SUPPORTS_ENUM = False

try:
    from collections.abc import Mapping, Sequence
except ImportError:
    from collections import Mapping, Sequence

import confuse
import os
import unittest
from . import _root


class ValidConfigTest(unittest.TestCase):
    def test_validate_simple_dict(self):
        config = _root({'foo': 5})
        valid = config.get({'foo': confuse.Integer()})
        self.assertEqual(valid['foo'], 5)

    def test_default_value(self):
        config = _root({})
        valid = config.get({'foo': confuse.Integer(8)})
        self.assertEqual(valid['foo'], 8)

    def test_undeclared_key_raises_keyerror(self):
        config = _root({'foo': 5})
        valid = config.get({'foo': confuse.Integer()})
        with self.assertRaises(KeyError):
            valid['bar']

    def test_undeclared_key_ignored_from_input(self):
        config = _root({'foo': 5, 'bar': 6})
        valid = config.get({'foo': confuse.Integer()})
        with self.assertRaises(KeyError):
            valid['bar']

    def test_int_template_shortcut(self):
        config = _root({'foo': 5})
        valid = config.get({'foo': int})
        self.assertEqual(valid['foo'], 5)

    def test_int_default_shortcut(self):
        config = _root({})
        valid = config.get({'foo': 9})
        self.assertEqual(valid['foo'], 9)

    def test_attribute_access(self):
        config = _root({'foo': 5})
        valid = config.get({'foo': confuse.Integer()})
        self.assertEqual(valid.foo, 5)

    def test_missing_required_value_raises_error_on_validate(self):
        config = _root({})
        with self.assertRaises(confuse.NotFoundError):
            config.get({'foo': confuse.Integer()})

    def test_none_as_default(self):
        config = _root({})
        valid = config.get({'foo': confuse.Integer(None)})
        self.assertIsNone(valid['foo'])

    def test_wrong_type_raises_error_on_validate(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.ConfigTypeError):
            config.get({'foo': confuse.Integer()})

    def test_validate_individual_value(self):
        config = _root({'foo': 5})
        valid = config['foo'].get(confuse.Integer())
        self.assertEqual(valid, 5)

    def test_nested_dict_template(self):
        config = _root({
            'foo': {'bar': 9},
        })
        valid = config.get({
            'foo': {'bar': confuse.Integer()},
        })
        self.assertEqual(valid['foo']['bar'], 9)

    def test_nested_attribute_access(self):
        config = _root({
            'foo': {'bar': 8},
        })
        valid = config.get({
            'foo': {'bar': confuse.Integer()},
        })
        self.assertEqual(valid.foo.bar, 8)


class AsTemplateTest(unittest.TestCase):
    def test_plain_int_as_template(self):
        typ = confuse.as_template(int)
        self.assertIsInstance(typ, confuse.Integer)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_concrete_int_as_template(self):
        typ = confuse.as_template(2)
        self.assertIsInstance(typ, confuse.Integer)
        self.assertEqual(typ.default, 2)

    def test_plain_string_as_template(self):
        typ = confuse.as_template(str)
        self.assertIsInstance(typ, confuse.String)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_concrete_string_as_template(self):
        typ = confuse.as_template('foo')
        self.assertIsInstance(typ, confuse.String)
        self.assertEqual(typ.default, 'foo')

    @unittest.skipIf(confuse.PY3, "unicode only present in Python 2")
    def test_unicode_type_as_template(self):
        typ = confuse.as_template(unicode)  # noqa ignore=F821
        self.assertIsInstance(typ, confuse.String)
        self.assertEqual(typ.default, confuse.REQUIRED)

    @unittest.skipIf(confuse.PY3, "basestring only present in Python 2")
    def test_basestring_as_template(self):
        typ = confuse.as_template(basestring)  # noqa ignore=F821
        self.assertIsInstance(typ, confuse.String)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_dict_as_template(self):
        typ = confuse.as_template({'key': 9})
        self.assertIsInstance(typ, confuse.MappingTemplate)
        self.assertIsInstance(typ.subtemplates['key'], confuse.Integer)
        self.assertEqual(typ.subtemplates['key'].default, 9)

    def test_nested_dict_as_template(self):
        typ = confuse.as_template({'outer': {'inner': 2}})
        self.assertIsInstance(typ, confuse.MappingTemplate)
        self.assertIsInstance(typ.subtemplates['outer'],
                              confuse.MappingTemplate)
        self.assertIsInstance(typ.subtemplates['outer'].subtemplates['inner'],
                              confuse.Integer)
        self.assertEqual(typ.subtemplates['outer'].subtemplates['inner']
                         .default, 2)

    def test_list_as_template(self):
        typ = confuse.as_template(list())
        self.assertIsInstance(typ, confuse.OneOf)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_set_as_template(self):
        typ = confuse.as_template(set())
        self.assertIsInstance(typ, confuse.Choice)

    @unittest.skipUnless(SUPPORTS_ENUM,
                         "enum not supported in this version of Python.")
    def test_enum_type_as_template(self):
        typ = confuse.as_template(enum.Enum)
        self.assertIsInstance(typ, confuse.Choice)

    def test_float_type_as_tempalte(self):
        typ = confuse.as_template(float)
        self.assertIsInstance(typ, confuse.Number)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_concrete_float_as_template(self):
        typ = confuse.as_template(2.)
        self.assertIsInstance(typ, confuse.Number)
        self.assertEqual(typ.default, 2.)

    def test_none_as_template(self):
        typ = confuse.as_template(None)
        self.assertIs(type(typ), confuse.Template)
        self.assertEqual(typ.default, None)

    def test_required_as_template(self):
        typ = confuse.as_template(confuse.REQUIRED)
        self.assertIs(type(typ), confuse.Template)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_dict_type_as_template(self):
        typ = confuse.as_template(dict)
        self.assertIsInstance(typ, confuse.TypeTemplate)
        self.assertEqual(typ.typ, Mapping)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_list_type_as_template(self):
        typ = confuse.as_template(list)
        self.assertIsInstance(typ, confuse.TypeTemplate)
        self.assertEqual(typ.typ, Sequence)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_set_type_as_template(self):
        typ = confuse.as_template(set)
        self.assertIsInstance(typ, confuse.TypeTemplate)
        self.assertEqual(typ.typ, set)
        self.assertEqual(typ.default, confuse.REQUIRED)

    def test_other_type_as_template(self):
        class MyClass(object):
            pass
        typ = confuse.as_template(MyClass)
        self.assertIsInstance(typ, confuse.TypeTemplate)
        self.assertEqual(typ.typ, MyClass)
        self.assertEqual(typ.default, confuse.REQUIRED)


class StringTemplateTest(unittest.TestCase):
    def test_validate_string(self):
        config = _root({'foo': 'bar'})
        valid = config.get({'foo': confuse.String()})
        self.assertEqual(valid['foo'], 'bar')

    def test_string_default_value(self):
        config = _root({})
        valid = config.get({'foo': confuse.String('baz')})
        self.assertEqual(valid['foo'], 'baz')

    def test_pattern_matching(self):
        config = _root({'foo': 'bar', 'baz': 'zab'})
        valid = config.get({'foo': confuse.String(pattern='^ba.$')})
        self.assertEqual(valid['foo'], 'bar')
        with self.assertRaises(confuse.ConfigValueError):
            config.get({'baz': confuse.String(pattern='!')})

    def test_string_template_shortcut(self):
        config = _root({'foo': 'bar'})
        valid = config.get({'foo': str})
        self.assertEqual(valid['foo'], 'bar')

    def test_string_default_shortcut(self):
        config = _root({})
        valid = config.get({'foo': 'bar'})
        self.assertEqual(valid['foo'], 'bar')

    def test_check_string_type(self):
        config = _root({'foo': 5})
        with self.assertRaises(confuse.ConfigTypeError):
            config.get({'foo': confuse.String()})


class NumberTest(unittest.TestCase):
    def test_validate_int_as_number(self):
        config = _root({'foo': 2})
        valid = config['foo'].get(confuse.Number())
        self.assertIsInstance(valid, int)
        self.assertEqual(valid, 2)

    def test_validate_float_as_number(self):
        config = _root({'foo': 3.0})
        valid = config['foo'].get(confuse.Number())
        self.assertIsInstance(valid, float)
        self.assertEqual(valid, 3.0)

    def test_validate_string_as_number(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.Number())


class ChoiceTest(unittest.TestCase):
    def test_validate_good_choice_in_list(self):
        config = _root({'foo': 2})
        valid = config['foo'].get(confuse.Choice([1, 2, 4, 8, 16]))
        self.assertEqual(valid, 2)

    def test_validate_bad_choice_in_list(self):
        config = _root({'foo': 3})
        with self.assertRaises(confuse.ConfigValueError):
            config['foo'].get(confuse.Choice([1, 2, 4, 8, 16]))

    def test_validate_good_choice_in_dict(self):
        config = _root({'foo': 2})
        valid = config['foo'].get(confuse.Choice({2: 'two', 4: 'four'}))
        self.assertEqual(valid, 'two')

    def test_validate_bad_choice_in_dict(self):
        config = _root({'foo': 3})
        with self.assertRaises(confuse.ConfigValueError):
            config['foo'].get(confuse.Choice({2: 'two', 4: 'four'}))


class OneOfTest(unittest.TestCase):
    def test_default_value(self):
        config = _root({})
        valid = config['foo'].get(confuse.OneOf([], default='bar'))
        self.assertEqual(valid, 'bar')

    def test_validate_good_choice_in_list(self):
        config = _root({'foo': 2})
        valid = config['foo'].get(confuse.OneOf([
            confuse.String(),
            confuse.Integer(),
        ]))
        self.assertEqual(valid, 2)

    def test_validate_first_good_choice_in_list(self):
        config = _root({'foo': 3.14})
        valid = config['foo'].get(confuse.OneOf([
            confuse.Integer(),
            confuse.Number(),
        ]))
        self.assertEqual(valid, 3)

    def test_validate_no_choice_in_list(self):
        config = _root({'foo': None})
        with self.assertRaises(confuse.ConfigValueError):
            config['foo'].get(confuse.OneOf([
                confuse.String(),
                confuse.Integer(),
            ]))

    def test_validate_bad_template(self):
        class BadTemplate(object):
            pass
        config = _root({})
        with self.assertRaises(confuse.ConfigTemplateError):
            config.get(confuse.OneOf([BadTemplate()]))
        del BadTemplate


class StrSeqTest(unittest.TestCase):
    def test_string_list(self):
        config = _root({'foo': ['bar', 'baz']})
        valid = config['foo'].get(confuse.StrSeq())
        self.assertEqual(valid, ['bar', 'baz'])

    def test_string_tuple(self):
        config = _root({'foo': ('bar', 'baz')})
        valid = config['foo'].get(confuse.StrSeq())
        self.assertEqual(valid, ['bar', 'baz'])

    def test_whitespace_separated_string(self):
        config = _root({'foo': 'bar   baz'})
        valid = config['foo'].get(confuse.StrSeq())
        self.assertEqual(valid, ['bar', 'baz'])

    def test_invalid_type(self):
        config = _root({'foo': 9})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.StrSeq())

    def test_invalid_sequence_type(self):
        config = _root({'foo': ['bar', 2126]})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.StrSeq())


class FilenameTest(unittest.TestCase):
    def test_default_value(self):
        config = _root({})
        valid = config['foo'].get(confuse.Filename('foo/bar'))
        self.assertEqual(valid, 'foo/bar')

    def test_default_none(self):
        config = _root({})
        valid = config['foo'].get(confuse.Filename(None))
        self.assertEqual(valid, None)

    def test_missing_required_value(self):
        config = _root({})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(confuse.Filename())

    def test_filename_relative_to_working_dir(self):
        config = _root({'foo': 'bar'})
        valid = config['foo'].get(confuse.Filename(cwd='/dev/null'))
        self.assertEqual(valid, os.path.realpath('/dev/null/bar'))

    def test_filename_relative_to_sibling(self):
        config = _root({'foo': '/', 'bar': 'baz'})
        valid = config.get({
            'foo': confuse.Filename(),
            'bar': confuse.Filename(relative_to='foo')
        })
        self.assertEqual(valid.foo, os.path.realpath('/'))
        self.assertEqual(valid.bar, os.path.realpath('/baz'))

    def test_filename_working_dir_overrides_sibling(self):
        config = _root({'foo': 'bar'})
        valid = config.get({
            'foo': confuse.Filename(cwd='/dev/null', relative_to='baz')
        })
        self.assertEqual(valid.foo, os.path.realpath('/dev/null/bar'))

    def test_filename_relative_to_sibling_with_recursion(self):
        config = _root({'foo': '/', 'bar': 'r', 'baz': 'z'})
        with self.assertRaises(confuse.ConfigTemplateError):
            config.get({
                'foo': confuse.Filename(relative_to='bar'),
                'bar': confuse.Filename(relative_to='baz'),
                'baz': confuse.Filename(relative_to='foo')
            })

    def test_filename_relative_to_self(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.ConfigTemplateError):
            config.get({
                'foo': confuse.Filename(relative_to='foo')
            })

    def test_filename_relative_to_sibling_needs_siblings(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.ConfigTemplateError):
            config['foo'].get(confuse.Filename(relative_to='bar'))

    def test_filename_relative_to_sibling_needs_template(self):
        config = _root({'foo': '/', 'bar': 'baz'})
        with self.assertRaises(confuse.ConfigTemplateError):
            config.get({
                'bar': confuse.Filename(relative_to='foo')
            })

    def test_filename_with_non_file_source(self):
        config = _root({'foo': 'foo/bar'})
        valid = config['foo'].get(confuse.Filename())
        self.assertEqual(valid, os.path.join(os.getcwd(), 'foo', 'bar'))

    def test_filename_with_file_source(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml')
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename())
        self.assertEqual(valid, os.path.realpath('/config/path/foo/bar'))

    def test_filename_with_default_source(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml',
                                      default=True)
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename())
        self.assertEqual(valid, os.path.realpath('/config/path/foo/bar'))

    def test_filename_use_config_source_dir(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml',
                                      base_for_paths=True)
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename())
        self.assertEqual(valid, os.path.realpath('/baz/foo/bar'))

    def test_filename_in_source_dir(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml')
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename(in_source_dir=True))
        self.assertEqual(valid, os.path.realpath('/baz/foo/bar'))

    def test_filename_in_source_dir_overrides_in_app_dir(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml')
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename(in_source_dir=True,
                                                   in_app_dir=True))
        self.assertEqual(valid, os.path.realpath('/baz/foo/bar'))

    def test_filename_in_app_dir_non_file_source(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'})
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename(in_app_dir=True))
        self.assertEqual(valid, os.path.realpath('/config/path/foo/bar'))

    def test_filename_in_app_dir_overrides_config_source_dir(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml',
                                      base_for_paths=True)
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        valid = config['foo'].get(confuse.Filename(in_app_dir=True))
        self.assertEqual(valid, os.path.realpath('/config/path/foo/bar'))

    def test_filename_wrong_type(self):
        config = _root({'foo': 8})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.Filename())


class PathTest(unittest.TestCase):
    def test_path_value(self):
        import pathlib
        config = _root({'foo': 'foo/bar'})
        valid = config['foo'].get(confuse.Path())
        self.assertEqual(valid, pathlib.Path(os.path.abspath('foo/bar')))

    def test_default_value(self):
        import pathlib
        config = _root({})
        valid = config['foo'].get(confuse.Path('foo/bar'))
        self.assertEqual(valid, pathlib.Path('foo/bar'))

    def test_default_none(self):
        config = _root({})
        valid = config['foo'].get(confuse.Path(None))
        self.assertEqual(valid, None)

    def test_missing_required_value(self):
        config = _root({})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(confuse.Path())


class BaseTemplateTest(unittest.TestCase):
    def test_base_template_accepts_any_value(self):
        config = _root({'foo': 4.2})
        valid = config['foo'].get(confuse.Template())
        self.assertEqual(valid, 4.2)

    def test_base_template_required(self):
        config = _root({})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(confuse.Template())

    def test_base_template_with_default(self):
        config = _root({})
        valid = config['foo'].get(confuse.Template('bar'))
        self.assertEqual(valid, 'bar')


class TypeTemplateTest(unittest.TestCase):
    def test_correct_type(self):
        config = _root({'foo': set()})
        valid = config['foo'].get(confuse.TypeTemplate(set))
        self.assertEqual(valid, set())

    def test_incorrect_type(self):
        config = _root({'foo': dict()})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.TypeTemplate(set))

    def test_missing_required_value(self):
        config = _root({})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(confuse.TypeTemplate(set))

    def test_default_value(self):
        config = _root({})
        valid = config['foo'].get(confuse.TypeTemplate(set, set([1, 2])))
        self.assertEqual(valid, set([1, 2]))


class SequenceTest(unittest.TestCase):
    def test_int_list(self):
        config = _root({'foo': [1, 2, 3]})
        valid = config['foo'].get(confuse.Sequence(int))
        self.assertEqual(valid, [1, 2, 3])

    def test_dict_list(self):
        config = _root({'foo': [{'bar': 1, 'baz': 2}, {'bar': 3, 'baz': 4}]})
        valid = config['foo'].get(confuse.Sequence(
            {'bar': int, 'baz': int}
        ))
        self.assertEqual(valid, [
            {'bar': 1, 'baz': 2}, {'bar': 3, 'baz': 4}
        ])

    def test_invalid_item(self):
        config = _root({'foo': [{'bar': 1, 'baz': 2}, {'bar': 3, 'bak': 4}]})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(confuse.Sequence(
                {'bar': int, 'baz': int}
            ))

    def test_wrong_type(self):
        config = _root({'foo': {'one': 1, 'two': 2, 'three': 3}})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.Sequence(int))

    def test_missing(self):
        config = _root({'foo': [1, 2, 3]})
        valid = config['bar'].get(confuse.Sequence(int))
        self.assertEqual(valid, [])


class MappingValuesTest(unittest.TestCase):
    def test_int_dict(self):
        config = _root({'foo': {'one': 1, 'two': 2, 'three': 3}})
        valid = config['foo'].get(confuse.MappingValues(int))
        self.assertEqual(valid, {'one': 1, 'two': 2, 'three': 3})

    def test_dict_dict(self):
        config = _root({'foo': {'first': {'bar': 1, 'baz': 2},
                                'second': {'bar': 3, 'baz': 4}}})
        valid = config['foo'].get(confuse.MappingValues(
            {'bar': int, 'baz': int}
        ))
        self.assertEqual(valid, {
            'first': {'bar': 1, 'baz': 2},
            'second': {'bar': 3, 'baz': 4},
        })

    def test_invalid_item(self):
        config = _root({'foo': {'first': {'bar': 1, 'baz': 2},
                                'second': {'bar': 3, 'bak': 4}}})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(confuse.MappingValues(
                {'bar': int, 'baz': int}
            ))

    def test_wrong_type(self):
        config = _root({'foo': [1, 2, 3]})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.MappingValues(int))

    def test_missing(self):
        config = _root({'foo': {'one': 1, 'two': 2, 'three': 3}})
        valid = config['bar'].get(confuse.MappingValues(int))
        self.assertEqual(valid, {})


class OptionalTest(unittest.TestCase):
    def test_optional_string_valid_type(self):
        config = _root({'foo': 'bar'})
        valid = config['foo'].get(confuse.Optional(confuse.String()))
        self.assertEqual(valid, 'bar')

    def test_optional_string_invalid_type(self):
        config = _root({'foo': 5})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(confuse.Optional(confuse.String()))

    def test_optional_string_null(self):
        config = _root({'foo': None})
        valid = config['foo'].get(confuse.Optional(confuse.String()))
        self.assertIsNone(valid)

    def test_optional_string_null_default_value(self):
        config = _root({'foo': None})
        valid = config['foo'].get(confuse.Optional(confuse.String(), 'baz'))
        self.assertEqual(valid, 'baz')

    def test_optional_string_null_string_provides_default(self):
        config = _root({'foo': None})
        valid = config['foo'].get(confuse.Optional(confuse.String('baz')))
        self.assertEqual(valid, 'baz')

    def test_optional_string_null_string_default_override(self):
        config = _root({'foo': None})
        valid = config['foo'].get(confuse.Optional(confuse.String('baz'),
                                                   default='bar'))
        self.assertEqual(valid, 'bar')

    def test_optional_string_allow_missing_no_explicit_default(self):
        config = _root({})
        valid = config['foo'].get(confuse.Optional(confuse.String()))
        self.assertIsNone(valid)

    def test_optional_string_allow_missing_default_value(self):
        config = _root({})
        valid = config['foo'].get(confuse.Optional(confuse.String(), 'baz'))
        self.assertEqual(valid, 'baz')

    def test_optional_string_missing_not_allowed(self):
        config = _root({})
        with self.assertRaises(confuse.NotFoundError):
            config['foo'].get(
                confuse.Optional(confuse.String(), allow_missing=False)
            )

    def test_optional_string_null_missing_not_allowed(self):
        config = _root({'foo': None})
        valid = config['foo'].get(
            confuse.Optional(confuse.String(), allow_missing=False)
        )
        self.assertIsNone(valid)

    def test_optional_mapping_template_valid(self):
        config = _root({'foo': {'bar': 5, 'baz': 'bak'}})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        valid = config.get({'foo': confuse.Optional(template)})
        self.assertEqual(valid['foo']['bar'], 5)
        self.assertEqual(valid['foo']['baz'], 'bak')

    def test_optional_mapping_template_invalid(self):
        config = _root({'foo': {'bar': 5, 'baz': 10}})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        with self.assertRaises(confuse.ConfigTypeError):
            config.get({'foo': confuse.Optional(template)})

    def test_optional_mapping_template_null(self):
        config = _root({'foo': None})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        valid = config.get({'foo': confuse.Optional(template)})
        self.assertIsNone(valid['foo'])

    def test_optional_mapping_template_null_default_value(self):
        config = _root({'foo': None})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        valid = config.get({'foo': confuse.Optional(template, {})})
        self.assertIsInstance(valid['foo'], dict)

    def test_optional_mapping_template_allow_missing_no_explicit_default(self):
        config = _root({})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        valid = config.get({'foo': confuse.Optional(template)})
        self.assertIsNone(valid['foo'])

    def test_optional_mapping_template_allow_missing_default_value(self):
        config = _root({})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        valid = config.get({'foo': confuse.Optional(template, {})})
        self.assertIsInstance(valid['foo'], dict)

    def test_optional_mapping_template_missing_not_allowed(self):
        config = _root({})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        with self.assertRaises(confuse.NotFoundError):
            config.get({'foo': confuse.Optional(template,
                                                allow_missing=False)})

    def test_optional_mapping_template_null_missing_not_allowed(self):
        config = _root({'foo': None})
        template = {'bar': confuse.Integer(), 'baz': confuse.String()}
        valid = config.get({'foo': confuse.Optional(template,
                                                    allow_missing=False)})
        self.assertIsNone(valid['foo'])



from __future__ import division, absolute_import, print_function

import confuse
import os
import unittest
from . import _root


ENVIRON = os.environ


class EnvSourceTest(unittest.TestCase):
    def setUp(self):
        os.environ = {}

    def tearDown(self):
        os.environ = ENVIRON

    def test_prefix(self):
        os.environ['TEST_FOO'] = 'a'
        os.environ['BAR'] = 'b'
        config = _root(confuse.EnvSource('TEST_'))
        self.assertEqual(config.get(), {'foo': 'a'})

    def test_number_type_conversion(self):
        os.environ['TEST_FOO'] = '1'
        os.environ['TEST_BAR'] = '2.0'
        config = _root(confuse.EnvSource('TEST_'))
        foo = config['foo'].get()
        bar = config['bar'].get()
        self.assertIsInstance(foo, int)
        self.assertEqual(foo, 1)
        self.assertIsInstance(bar, float)
        self.assertEqual(bar, 2.0)

    def test_bool_type_conversion(self):
        os.environ['TEST_FOO'] = 'true'
        os.environ['TEST_BAR'] = 'FALSE'
        config = _root(confuse.EnvSource('TEST_'))
        self.assertIs(config['foo'].get(), True)
        self.assertIs(config['bar'].get(), False)

    def test_null_type_conversion(self):
        os.environ['TEST_FOO'] = 'null'
        os.environ['TEST_BAR'] = ''
        config = _root(confuse.EnvSource('TEST_'))
        self.assertIs(config['foo'].get(), None)
        self.assertIs(config['bar'].get(), None)

    def test_unset_lower_config(self):
        os.environ['TEST_FOO'] = 'null'
        config = _root({'foo': 'bar'})
        self.assertEqual(config['foo'].get(), 'bar')
        config.set(confuse.EnvSource('TEST_'))
        self.assertIs(config['foo'].get(), None)

    def test_sep_default(self):
        os.environ['TEST_FOO__BAR'] = 'a'
        os.environ['TEST_FOO_BAZ'] = 'b'
        config = _root(confuse.EnvSource('TEST_'))
        self.assertEqual(config['foo']['bar'].get(), 'a')
        self.assertEqual(config['foo_baz'].get(), 'b')

    def test_sep_single_underscore_adjacent_seperators(self):
        os.environ['TEST_FOO__BAR'] = 'a'
        os.environ['TEST_FOO_BAZ'] = 'b'
        config = _root(confuse.EnvSource('TEST_', sep='_'))
        self.assertEqual(config['foo']['']['bar'].get(), 'a')
        self.assertEqual(config['foo']['baz'].get(), 'b')

    def test_nested(self):
        os.environ['TEST_FOO__BAR'] = 'a'
        os.environ['TEST_FOO__BAZ__QUX'] = 'b'
        config = _root(confuse.EnvSource('TEST_'))
        self.assertEqual(config['foo']['bar'].get(), 'a')
        self.assertEqual(config['foo']['baz']['qux'].get(), 'b')

    def test_nested_rev(self):
        # Reverse to ensure order doesn't matter
        os.environ['TEST_FOO__BAZ__QUX'] = 'b'
        os.environ['TEST_FOO__BAR'] = 'a'
        config = _root(confuse.EnvSource('TEST_'))
        self.assertEqual(config['foo']['bar'].get(), 'a')
        self.assertEqual(config['foo']['baz']['qux'].get(), 'b')

    def test_nested_clobber(self):
        os.environ['TEST_FOO__BAR'] = 'a'
        os.environ['TEST_FOO__BAR__BAZ'] = 'b'
        config = _root(confuse.EnvSource('TEST_'))
        # Clobbered
        self.assertEqual(config['foo']['bar'].get(), {'baz': 'b'})
        self.assertEqual(config['foo']['bar']['baz'].get(), 'b')

    def test_nested_clobber_rev(self):
        # Reverse to ensure order doesn't matter
        os.environ['TEST_FOO__BAR__BAZ'] = 'b'
        os.environ['TEST_FOO__BAR'] = 'a'
        config = _root(confuse.EnvSource('TEST_'))
        # Clobbered
        self.assertEqual(config['foo']['bar'].get(), {'baz': 'b'})
        self.assertEqual(config['foo']['bar']['baz'].get(), 'b')

    def test_lower_applied_after_prefix_match(self):
        os.environ['TEST_FOO'] = 'a'
        config = _root(confuse.EnvSource('test_', lower=True))
        self.assertEqual(config.get(), {})

    def test_lower_already_lowercase(self):
        os.environ['TEST_foo'] = 'a'
        config = _root(confuse.EnvSource('TEST_', lower=True))
        self.assertEqual(config.get(), {'foo': 'a'})

    def test_lower_does_not_alter_value(self):
        os.environ['TEST_FOO'] = 'UPPER'
        config = _root(confuse.EnvSource('TEST_', lower=True))
        self.assertEqual(config.get(), {'foo': 'UPPER'})

    def test_lower_false(self):
        os.environ['TEST_FOO'] = 'a'
        config = _root(confuse.EnvSource('TEST_', lower=False))
        self.assertEqual(config.get(), {'FOO': 'a'})

    def test_handle_lists_good_list(self):
        os.environ['TEST_FOO__0'] = 'a'
        os.environ['TEST_FOO__1'] = 'b'
        os.environ['TEST_FOO__2'] = 'c'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), ['a', 'b', 'c'])

    def test_handle_lists_good_list_rev(self):
        # Reverse to ensure order doesn't matter
        os.environ['TEST_FOO__2'] = 'c'
        os.environ['TEST_FOO__1'] = 'b'
        os.environ['TEST_FOO__0'] = 'a'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), ['a', 'b', 'c'])

    def test_handle_lists_nested_lists(self):
        os.environ['TEST_FOO__0__0'] = 'a'
        os.environ['TEST_FOO__0__1'] = 'b'
        os.environ['TEST_FOO__1__0'] = 'c'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), [['a', 'b'], ['c']])

    def test_handle_lists_bad_list_missing_index(self):
        os.environ['TEST_FOO__0'] = 'a'
        os.environ['TEST_FOO__2'] = 'b'
        os.environ['TEST_FOO__3'] = 'c'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), {'0': 'a', '2': 'b', '3': 'c'})

    def test_handle_lists_bad_list_non_zero_start(self):
        os.environ['TEST_FOO__1'] = 'a'
        os.environ['TEST_FOO__2'] = 'b'
        os.environ['TEST_FOO__3'] = 'c'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), {'1': 'a', '2': 'b', '3': 'c'})

    def test_handle_lists_bad_list_non_numeric(self):
        os.environ['TEST_FOO__0'] = 'a'
        os.environ['TEST_FOO__ONE'] = 'b'
        os.environ['TEST_FOO__2'] = 'c'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), {'0': 'a', 'one': 'b', '2': 'c'})

    def test_handle_lists_top_level_always_dict(self):
        os.environ['TEST_0'] = 'a'
        os.environ['TEST_1'] = 'b'
        os.environ['TEST_2'] = 'c'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config.get(), {'0': 'a', '1': 'b', '2': 'c'})

    def test_handle_lists_not_a_list(self):
        os.environ['TEST_FOO__BAR'] = 'a'
        os.environ['TEST_FOO__BAZ'] = 'b'
        config = _root(confuse.EnvSource('TEST_', handle_lists=True))
        self.assertEqual(config['foo'].get(), {'bar': 'a', 'baz': 'b'})

    def test_parse_yaml_docs_scalar(self):
        os.environ['TEST_FOO'] = 'a'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        self.assertEqual(config['foo'].get(), 'a')

    def test_parse_yaml_docs_list(self):
        os.environ['TEST_FOO'] = '[a, b]'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        self.assertEqual(config['foo'].get(), ['a', 'b'])

    def test_parse_yaml_docs_dict(self):
        os.environ['TEST_FOO'] = '{bar: a, baz: b}'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        self.assertEqual(config['foo'].get(), {'bar': 'a', 'baz': 'b'})

    def test_parse_yaml_docs_nested(self):
        os.environ['TEST_FOO'] = '{bar: [a, b], baz: {qux: c}}'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        self.assertEqual(config['foo']['bar'].get(), ['a', 'b'])
        self.assertEqual(config['foo']['baz'].get(), {'qux': 'c'})

    def test_parse_yaml_docs_number_conversion(self):
        os.environ['TEST_FOO'] = '{bar: 1, baz: 2.0}'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        bar = config['foo']['bar'].get()
        baz = config['foo']['baz'].get()
        self.assertIsInstance(bar, int)
        self.assertEqual(bar, 1)
        self.assertIsInstance(baz, float)
        self.assertEqual(baz, 2.0)

    def test_parse_yaml_docs_bool_conversion(self):
        os.environ['TEST_FOO'] = '{bar: true, baz: FALSE}'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        self.assertIs(config['foo']['bar'].get(), True)
        self.assertIs(config['foo']['baz'].get(), False)

    def test_parse_yaml_docs_null_conversion(self):
        os.environ['TEST_FOO'] = '{bar: null, baz: }'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        self.assertIs(config['foo']['bar'].get(), None)
        self.assertIs(config['foo']['baz'].get(), None)

    def test_parse_yaml_docs_syntax_error(self):
        os.environ['TEST_FOO'] = '{:}'
        try:
            _root(confuse.EnvSource('TEST_', parse_yaml_docs=True))
        except confuse.ConfigError as exc:
            self.assertTrue('TEST_FOO' in exc.name)
        else:
            self.fail('ConfigError not raised')

    def test_parse_yaml_docs_false(self):
        os.environ['TEST_FOO'] = '{bar: a, baz: b}'
        config = _root(confuse.EnvSource('TEST_', parse_yaml_docs=False))
        self.assertEqual(config['foo'].get(), '{bar: a, baz: b}')
        with self.assertRaises(confuse.ConfigError):
            config['foo']['bar'].get()


class ConfigEnvTest(unittest.TestCase):
    def setUp(self):
        self.config = confuse.Configuration('TestApp', read=False)
        os.environ = {
            'TESTAPP_FOO': 'a',
            'TESTAPP_BAR__NESTED': 'b',
            'TESTAPP_BAZ_SEP_NESTED': 'c',
            'MYAPP_QUX_SEP_NESTED': 'd'
        }

    def tearDown(self):
        os.environ = ENVIRON

    def test_defaults(self):
        self.config.set_env()
        self.assertEqual(self.config.get(), {'foo': 'a',
                                             'bar': {'nested': 'b'},
                                             'baz_sep_nested': 'c'})

    def test_with_prefix(self):
        self.config.set_env(prefix='MYAPP_')
        self.assertEqual(self.config.get(), {'qux_sep_nested': 'd'})

    def test_with_sep(self):
        self.config.set_env(sep='_sep_')
        self.assertEqual(self.config.get(), {'foo': 'a',
                                             'bar__nested': 'b',
                                             'baz': {'nested': 'c'}})

    def test_with_prefix_and_sep(self):
        self.config.set_env(prefix='MYAPP_', sep='_sep_')
        self.assertEqual(self.config.get(), {'qux': {'nested': 'd'}})



import os
import sys
import datetime as dt


sys.path.insert(0, os.path.abspath(".."))

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx',
    'sphinx.ext.viewcode',
    'sphinx.ext.autosectionlabel',
    'sphinx_rtd_theme',
]
source_suffix = '.rst'
master_doc = 'index'

project = u'Confuse'
copyright = '2012-{}, Adrian Sampson & contributors'.format(
    dt.date.today().year
)

version = '1.7'
release = '1.7.0'

exclude_patterns = ['_build']

pygments_style = 'sphinx'

# -- Options for HTML output --------------------------------------------------

html_theme = 'sphinx_rtd_theme'
htmlhelp_basename = 'Confusedoc'



from __future__ import division, absolute_import, print_function

import confuse
import textwrap
import unittest
from . import _root


class PrettyDumpTest(unittest.TestCase):
    def test_dump_null(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': None})
        yaml = config.dump().strip()
        self.assertEqual(yaml, 'foo:')

    def test_dump_true(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': True})
        yaml = config.dump().strip()
        self.assertEqual(yaml, 'foo: yes')

    def test_dump_false(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': False})
        yaml = config.dump().strip()
        self.assertEqual(yaml, 'foo: no')

    def test_dump_short_list(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': ['bar', 'baz']})
        yaml = config.dump().strip()
        self.assertEqual(yaml, 'foo: [bar, baz]')

    def test_dump_ordered_dict(self):
        odict = confuse.OrderedDict()
        odict['foo'] = 'bar'
        odict['bar'] = 'baz'
        odict['baz'] = 'qux'

        config = confuse.Configuration('myapp', read=False)
        config.add({'key': odict})
        yaml = config.dump().strip()
        self.assertEqual(yaml, textwrap.dedent("""
            key:
                foo: bar
                bar: baz
                baz: qux
        """).strip())

    def test_dump_sans_defaults(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': 'bar'})
        config.sources[0].default = True
        config.add({'baz': 'qux'})

        yaml = config.dump().strip()
        self.assertEqual(yaml, "foo: bar\nbaz: qux")

        yaml = config.dump(full=False).strip()
        self.assertEqual(yaml, "baz: qux")


class RedactTest(unittest.TestCase):
    def test_no_redaction(self):
        config = _root({'foo': 'bar'})
        data = config.flatten(redact=True)
        self.assertEqual(data, {'foo': 'bar'})

    def test_redact_key(self):
        config = _root({'foo': 'bar'})
        config['foo'].redact = True
        data = config.flatten(redact=True)
        self.assertEqual(data, {'foo': 'REDACTED'})

    def test_unredact(self):
        config = _root({'foo': 'bar'})
        config['foo'].redact = True
        config['foo'].redact = False
        data = config.flatten(redact=True)
        self.assertEqual(data, {'foo': 'bar'})

    def test_dump_redacted(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': 'bar'})
        config['foo'].redact = True
        yaml = config.dump(redact=True).strip()
        self.assertEqual(yaml, 'foo: REDACTED')

    def test_dump_unredacted(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': 'bar'})
        config['foo'].redact = True
        yaml = config.dump(redact=False).strip()
        self.assertEqual(yaml, 'foo: bar')

    def test_dump_redacted_sans_defaults(self):
        config = confuse.Configuration('myapp', read=False)
        config.add({'foo': 'bar'})
        config.sources[0].default = True
        config.add({'baz': 'qux'})
        config['baz'].redact = True

        yaml = config.dump(redact=True, full=False).strip()
        self.assertEqual(yaml, "baz: REDACTED")



from __future__ import division, absolute_import, print_function

from argparse import Namespace
from collections import OrderedDict
import confuse
import unittest


class BuildDictTests(unittest.TestCase):
    def test_pure_dicts(self):
        config = {'foo': {'bar': 1}}
        result = confuse.util.build_dict(config)
        self.assertEqual(1, result['foo']['bar'])

    def test_namespaces(self):
        config = Namespace(foo=Namespace(bar=2), another=1)
        result = confuse.util.build_dict(config)
        self.assertEqual(2, result['foo']['bar'])
        self.assertEqual(1, result['another'])

    def test_dot_sep_keys(self):
        config = {'foo.bar': 1}
        result = confuse.util.build_dict(config.copy())
        self.assertEqual(1, result['foo.bar'])

        result = confuse.util.build_dict(config.copy(), sep='.')
        self.assertEqual(1, result['foo']['bar'])

    def test_dot_sep_keys_clobber(self):
        args = [('foo.bar', 1), ('foo.bar.zar', 2)]
        config = OrderedDict(args)
        result = confuse.util.build_dict(config.copy(), sep='.')
        self.assertEqual({'zar': 2}, result['foo']['bar'])
        self.assertEqual(2, result['foo']['bar']['zar'])

        # Reverse and do it again! (should be stable)
        args.reverse()
        config = OrderedDict(args)
        result = confuse.util.build_dict(config.copy(), sep='.')
        self.assertEqual({'zar': 2}, result['foo']['bar'])
        self.assertEqual(2, result['foo']['bar']['zar'])

    def test_dot_sep_keys_no_clobber(self):
        args = [('foo.bar', 1), ('foo.far', 2), ('foo.zar.dar', 4)]
        config = OrderedDict(args)
        result = confuse.util.build_dict(config.copy(), sep='.')
        self.assertEqual(1, result['foo']['bar'])
        self.assertEqual(2, result['foo']['far'])
        self.assertEqual(4, result['foo']['zar']['dar'])

    def test_adjacent_underscores_sep_keys(self):
        config = {'foo__bar_baz': 1}
        result = confuse.util.build_dict(config.copy())
        self.assertEqual(1, result['foo__bar_baz'])

        result = confuse.util.build_dict(config.copy(), sep='_')
        self.assertEqual(1, result['foo']['']['bar']['baz'])

        result = confuse.util.build_dict(config.copy(), sep='__')
        self.assertEqual(1, result['foo']['bar_baz'])

    def test_keep_none(self):
        config = {'foo': None}
        result = confuse.util.build_dict(config.copy())
        with self.assertRaises(KeyError):
            result['foo']

        result = confuse.util.build_dict(config.copy(), keep_none=True)
        self.assertIs(None, result['foo'])

    def test_keep_none_with_nested(self):
        config = {'foo': {'bar': None}}
        result = confuse.util.build_dict(config.copy())
        self.assertEqual({}, result['foo'])

        result = confuse.util.build_dict(config.copy(), keep_none=True)
        self.assertIs(None, result['foo']['bar'])



from __future__ import division, absolute_import, print_function

import confuse
import sys
import unittest
from . import _root

PY3 = sys.version_info[0] == 3


class SingleSourceTest(unittest.TestCase):
    def test_dict_access(self):
        config = _root({'foo': 'bar'})
        value = config['foo'].get()
        self.assertEqual(value, 'bar')

    def test_list_access(self):
        config = _root({'foo': ['bar', 'baz']})
        value = config['foo'][1].get()
        self.assertEqual(value, 'baz')

    def test_missing_key(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.NotFoundError):
            config['baz'].get()

    def test_missing_index(self):
        config = _root({'l': ['foo', 'bar']})
        with self.assertRaises(confuse.NotFoundError):
            config['l'][5].get()

    def test_dict_iter(self):
        config = _root({'foo': 'bar', 'baz': 'qux'})
        keys = [key for key in config]
        self.assertEqual(set(keys), set(['foo', 'baz']))

    def test_list_iter(self):
        config = _root({'l': ['foo', 'bar']})
        items = [subview.get() for subview in config['l']]
        self.assertEqual(items, ['foo', 'bar'])

    def test_int_iter(self):
        config = _root({'n': 2})
        with self.assertRaises(confuse.ConfigTypeError):
            [item for item in config['n']]

    def test_dict_keys(self):
        config = _root({'foo': 'bar', 'baz': 'qux'})
        keys = config.keys()
        self.assertEqual(set(keys), set(['foo', 'baz']))

    def test_dict_values(self):
        config = _root({'foo': 'bar', 'baz': 'qux'})
        values = [value.get() for value in config.values()]
        self.assertEqual(set(values), set(['bar', 'qux']))

    def test_dict_items(self):
        config = _root({'foo': 'bar', 'baz': 'qux'})
        items = [(key, value.get()) for (key, value) in config.items()]
        self.assertEqual(set(items), set([('foo', 'bar'), ('baz', 'qux')]))

    def test_list_keys_error(self):
        config = _root({'l': ['foo', 'bar']})
        with self.assertRaises(confuse.ConfigTypeError):
            config['l'].keys()

    def test_list_sequence(self):
        config = _root({'l': ['foo', 'bar']})
        items = [item.get() for item in config['l'].sequence()]
        self.assertEqual(items, ['foo', 'bar'])

    def test_dict_sequence_error(self):
        config = _root({'foo': 'bar', 'baz': 'qux'})
        with self.assertRaises(confuse.ConfigTypeError):
            list(config.sequence())

    def test_dict_contents(self):
        config = _root({'foo': 'bar', 'baz': 'qux'})
        contents = config.all_contents()
        self.assertEqual(set(contents), set(['foo', 'baz']))

    def test_list_contents(self):
        config = _root({'l': ['foo', 'bar']})
        contents = config['l'].all_contents()
        self.assertEqual(list(contents), ['foo', 'bar'])

    def test_int_contents(self):
        config = _root({'n': 2})
        with self.assertRaises(confuse.ConfigTypeError):
            list(config['n'].all_contents())


class ConverstionTest(unittest.TestCase):
    def test_str_conversion_from_str(self):
        config = _root({'foo': 'bar'})
        value = str(config['foo'])
        self.assertEqual(value, 'bar')

    def test_str_conversion_from_int(self):
        config = _root({'foo': 2})
        value = str(config['foo'])
        self.assertEqual(value, '2')

    @unittest.skipIf(confuse.PY3, "unicode only present in Python 2")
    def test_unicode_conversion_from_int(self):
        config = _root({'foo': 2})
        value = unicode(config['foo'])  # noqa ignore=F821
        self.assertEqual(value, unicode('2'))  # noqa ignore=F821

    def test_bool_conversion_from_bool(self):
        config = _root({'foo': True})
        value = bool(config['foo'])
        self.assertEqual(value, True)

    def test_bool_conversion_from_int(self):
        config = _root({'foo': 0})
        value = bool(config['foo'])
        self.assertEqual(value, False)


class NameTest(unittest.TestCase):
    def test_root_name(self):
        config = _root()
        self.assertEqual(config.name, 'root')

    def test_string_access_name(self):
        config = _root()
        name = config['foo'].name
        self.assertEqual(name, "foo")

    def test_int_access_name(self):
        config = _root()
        name = config[5].name
        self.assertEqual(name, "#5")

    def test_nested_access_name(self):
        config = _root()
        name = config[5]['foo']['bar'][20].name
        self.assertEqual(name, "#5.foo.bar#20")


class MultipleSourceTest(unittest.TestCase):
    def test_dict_access_shadowed(self):
        config = _root({'foo': 'bar'}, {'foo': 'baz'})
        value = config['foo'].get()
        self.assertEqual(value, 'bar')

    def test_dict_access_fall_through(self):
        config = _root({'qux': 'bar'}, {'foo': 'baz'})
        value = config['foo'].get()
        self.assertEqual(value, 'baz')

    def test_dict_access_missing(self):
        config = _root({'qux': 'bar'}, {'foo': 'baz'})
        with self.assertRaises(confuse.NotFoundError):
            config['fred'].get()

    def test_list_access_shadowed(self):
        config = _root({'l': ['a', 'b']}, {'l': ['c', 'd', 'e']})
        value = config['l'][1].get()
        self.assertEqual(value, 'b')

    def test_list_access_fall_through(self):
        config = _root({'l': ['a', 'b']}, {'l': ['c', 'd', 'e']})
        value = config['l'][2].get()
        self.assertEqual(value, 'e')

    def test_list_access_missing(self):
        config = _root({'l': ['a', 'b']}, {'l': ['c', 'd', 'e']})
        with self.assertRaises(confuse.NotFoundError):
            config['l'][3].get()

    def test_access_dict_replaced(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'qux': 'fred'}})
        value = config['foo'].get()
        self.assertEqual(value, {'bar': 'baz'})

    def test_dict_keys_merged(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'qux': 'fred'}})
        keys = config['foo'].keys()
        self.assertEqual(set(keys), set(['bar', 'qux']))

    def test_dict_keys_replaced(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'bar': 'fred'}})
        keys = config['foo'].keys()
        self.assertEqual(list(keys), ['bar'])

    def test_dict_values_merged(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'qux': 'fred'}})
        values = [value.get() for value in config['foo'].values()]
        self.assertEqual(set(values), set(['baz', 'fred']))

    def test_dict_values_replaced(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'bar': 'fred'}})
        values = [value.get() for value in config['foo'].values()]
        self.assertEqual(list(values), ['baz'])

    def test_dict_items_merged(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'qux': 'fred'}})
        items = [(key, value.get()) for (key, value) in config['foo'].items()]
        self.assertEqual(set(items), set([('bar', 'baz'), ('qux', 'fred')]))

    def test_dict_items_replaced(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'bar': 'fred'}})
        items = [(key, value.get()) for (key, value) in config['foo'].items()]
        self.assertEqual(list(items), [('bar', 'baz')])

    def test_list_sequence_shadowed(self):
        config = _root({'l': ['a', 'b']}, {'l': ['c', 'd', 'e']})
        items = [item.get() for item in config['l'].sequence()]
        self.assertEqual(items, ['a', 'b'])

    def test_list_sequence_shadowed_by_dict(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': ['qux', 'fred']})
        with self.assertRaises(confuse.ConfigTypeError):
            list(config['foo'].sequence())

    def test_dict_contents_concatenated(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'qux': 'fred'}})
        contents = config['foo'].all_contents()
        self.assertEqual(set(contents), set(['bar', 'qux']))

    def test_dict_contents_concatenated_not_replaced(self):
        config = _root({'foo': {'bar': 'baz'}}, {'foo': {'bar': 'fred'}})
        contents = config['foo'].all_contents()
        self.assertEqual(list(contents), ['bar', 'bar'])

    def test_list_contents_concatenated(self):
        config = _root({'foo': ['bar', 'baz']}, {'foo': ['qux', 'fred']})
        contents = config['foo'].all_contents()
        self.assertEqual(list(contents), ['bar', 'baz', 'qux', 'fred'])

    def test_int_contents_error(self):
        config = _root({'foo': ['bar', 'baz']}, {'foo': 5})
        with self.assertRaises(confuse.ConfigTypeError):
            list(config['foo'].all_contents())

    def test_list_and_dict_contents_concatenated(self):
        config = _root({'foo': ['bar', 'baz']}, {'foo': {'qux': 'fred'}})
        contents = config['foo'].all_contents()
        self.assertEqual(list(contents), ['bar', 'baz', 'qux'])

    def test_add_source(self):
        config = _root({'foo': 'bar'})
        config.add({'baz': 'qux'})
        self.assertEqual(config['foo'].get(), 'bar')
        self.assertEqual(config['baz'].get(), 'qux')


class SetTest(unittest.TestCase):
    def test_set_missing_top_level_key(self):
        config = _root({})
        config['foo'] = 'bar'
        self.assertEqual(config['foo'].get(), 'bar')

    def test_override_top_level_key(self):
        config = _root({'foo': 'bar'})
        config['foo'] = 'baz'
        self.assertEqual(config['foo'].get(), 'baz')

    def test_set_second_level_key(self):
        config = _root({})
        config['foo']['bar'] = 'baz'
        self.assertEqual(config['foo']['bar'].get(), 'baz')

    def test_override_second_level_key(self):
        config = _root({'foo': {'bar': 'qux'}})
        config['foo']['bar'] = 'baz'
        self.assertEqual(config['foo']['bar'].get(), 'baz')

    def test_override_list_index(self):
        config = _root({'foo': ['a', 'b', 'c']})
        config['foo'][1] = 'bar'
        self.assertEqual(config['foo'][1].get(), 'bar')



from __future__ import division, absolute_import, print_function

import confuse
import argparse
from argparse import Namespace
import optparse
import unittest


class ArgparseTest(unittest.TestCase):
    def setUp(self):
        self.config = confuse.Configuration('test', read=False)
        self.parser = argparse.ArgumentParser()

    def _parse(self, args, **kwargs):
        args = self.parser.parse_args(args.split())
        self.config.set_args(args, **kwargs)

    def test_text_argument_parsed(self):
        self.parser.add_argument('--foo', metavar='BAR')
        self._parse('--foo bar')
        self.assertEqual(self.config['foo'].get(), 'bar')

    def test_boolean_argument_parsed(self):
        self.parser.add_argument('--foo', action='store_true')
        self._parse('--foo')
        self.assertEqual(self.config['foo'].get(), True)

    def test_missing_optional_argument_not_included(self):
        self.parser.add_argument('--foo', metavar='BAR')
        self._parse('')
        with self.assertRaises(confuse.NotFoundError):
            self.config['foo'].get()

    def test_argument_overrides_default(self):
        self.config.add({'foo': 'baz'})

        self.parser.add_argument('--foo', metavar='BAR')
        self._parse('--foo bar')
        self.assertEqual(self.config['foo'].get(), 'bar')

    def test_nested_destination_single(self):
        self.parser.add_argument('--one', dest='one.foo')
        self.parser.add_argument('--two', dest='one.two.foo')
        self._parse('--two TWO', dots=True)
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')

    def test_nested_destination_nested(self):
        self.parser.add_argument('--one', dest='one.foo')
        self.parser.add_argument('--two', dest='one.two.foo')
        self._parse('--two TWO --one ONE', dots=True)
        self.assertEqual(self.config['one']['foo'].get(), 'ONE')
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')

    def test_nested_destination_nested_rev(self):
        self.parser.add_argument('--one', dest='one.foo')
        self.parser.add_argument('--two', dest='one.two.foo')
        # Reverse to ensure order doesn't matter
        self._parse('--one ONE --two TWO', dots=True)
        self.assertEqual(self.config['one']['foo'].get(), 'ONE')
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')

    def test_nested_destination_clobber(self):
        self.parser.add_argument('--one', dest='one.two')
        self.parser.add_argument('--two', dest='one.two.foo')
        self._parse('--two TWO --one ONE', dots=True)
        # Clobbered
        self.assertEqual(self.config['one']['two'].get(), {'foo': 'TWO'})
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')

    def test_nested_destination_clobber_rev(self):
        # Reversed order
        self.parser.add_argument('--two', dest='one.two.foo')
        self.parser.add_argument('--one', dest='one.two')
        self._parse('--one ONE --two TWO', dots=True)
        # Clobbered just the same
        self.assertEqual(self.config['one']['two'].get(), {'foo': 'TWO'})
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')


class OptparseTest(unittest.TestCase):
    def setUp(self):
        self.config = confuse.Configuration('test', read=False)
        self.parser = optparse.OptionParser()

    def _parse(self, args, **kwargs):
        options, _ = self.parser.parse_args(args.split())
        self.config.set_args(options, **kwargs)

    def test_text_argument_parsed(self):
        self.parser.add_option('--foo', metavar='BAR')
        self._parse('--foo bar')
        self.assertEqual(self.config['foo'].get(), 'bar')

    def test_boolean_argument_parsed(self):
        self.parser.add_option('--foo', action='store_true')
        self._parse('--foo')
        self.assertEqual(self.config['foo'].get(), True)

    def test_missing_optional_argument_not_included(self):
        self.parser.add_option('--foo', metavar='BAR')
        self._parse('')
        with self.assertRaises(confuse.NotFoundError):
            self.config['foo'].get()

    def test_argument_overrides_default(self):
        self.config.add({'foo': 'baz'})

        self.parser.add_option('--foo', metavar='BAR')
        self._parse('--foo bar')
        self.assertEqual(self.config['foo'].get(), 'bar')

    def test_nested_destination_single(self):
        self.parser.add_option('--one', dest='one.foo')
        self.parser.add_option('--two', dest='one.two.foo')
        self._parse('--two TWO', dots=True)
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')

    def test_nested_destination_nested(self):
        self.parser.add_option('--one', dest='one.foo')
        self.parser.add_option('--two', dest='one.two.foo')
        self._parse('--two TWO --one ONE', dots=True)
        self.assertEqual(self.config['one']['foo'].get(), 'ONE')
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')

    def test_nested_destination_nested_rev(self):
        self.parser.add_option('--one', dest='one.foo')
        self.parser.add_option('--two', dest='one.two.foo')
        # Reverse to ensure order doesn't matter
        self._parse('--one ONE --two TWO', dots=True)
        self.assertEqual(self.config['one']['foo'].get(), 'ONE')
        self.assertEqual(self.config['one']['two']['foo'].get(), 'TWO')


class GenericNamespaceTest(unittest.TestCase):
    def setUp(self):
        self.config = confuse.Configuration('test', read=False)

    def test_value_added_to_root(self):
        self.config.set_args(Namespace(foo='bar'))
        self.assertEqual(self.config['foo'].get(), 'bar')

    def test_value_added_to_subview(self):
        self.config['baz'].set_args(Namespace(foo='bar'))
        self.assertEqual(self.config['baz']['foo'].get(), 'bar')

    def test_nested_namespace(self):
        args = Namespace(
            first="Hello",
            nested=Namespace(
                second="World"
            )
        )
        self.config.set_args(args, dots=True)
        self.assertEqual(self.config['first'].get(), 'Hello')
        self.assertEqual(self.config['nested']['second'].get(), 'World')



from __future__ import division, absolute_import, print_function

import confuse
import confuse.yaml_util
import ntpath
import os
import platform
import posixpath
import shutil
import tempfile
import unittest


DEFAULT = [platform.system, os.environ, os.path]

SYSTEMS = {
    'Linux': [{'HOME': '/home/test', 'XDG_CONFIG_HOME': '~/xdgconfig'},
              posixpath],
    'Darwin': [{'HOME': '/Users/test'}, posixpath],
    'Windows': [{
        'APPDATA': '~\\winconfig',
        'HOME': 'C:\\Users\\test',
        'USERPROFILE': 'C:\\Users\\test',
    }, ntpath]
}


def _touch(path):
    open(path, 'a').close()


class FakeSystem(unittest.TestCase):
    SYS_NAME = None
    TMP_HOME = False

    def setUp(self):
        if self.SYS_NAME in SYSTEMS:
            self.os_path = os.path
            os.environ = {}

            environ, os.path = SYSTEMS[self.SYS_NAME]
            os.environ.update(environ)  # copy
            platform.system = lambda: self.SYS_NAME

        if self.TMP_HOME:
            self.home = tempfile.mkdtemp()
            os.environ['HOME'] = self.home
            os.environ['USERPROFILE'] = self.home

    def tearDown(self):
        platform.system, os.environ, os.path = DEFAULT
        if hasattr(self, 'home'):
            shutil.rmtree(self.home)


class LinuxTestCases(FakeSystem):
    SYS_NAME = 'Linux'

    def test_both_xdg_and_fallback_dirs(self):
        self.assertEqual(confuse.config_dirs(),
                         ['/home/test/.config', '/home/test/xdgconfig',
                          '/etc/xdg', '/etc'])

    def test_fallback_only(self):
        del os.environ['XDG_CONFIG_HOME']
        self.assertEqual(confuse.config_dirs(), ['/home/test/.config',
                                                 '/etc/xdg', '/etc'])

    def test_xdg_matching_fallback_not_duplicated(self):
        os.environ['XDG_CONFIG_HOME'] = '~/.config'
        self.assertEqual(confuse.config_dirs(), ['/home/test/.config',
                                                 '/etc/xdg', '/etc'])

    def test_xdg_config_dirs(self):
        os.environ['XDG_CONFIG_DIRS'] = '/usr/local/etc/xdg:/etc/xdg'
        self.assertEqual(confuse.config_dirs(), ['/home/test/.config',
                                                 '/home/test/xdgconfig',
                                                 '/usr/local/etc/xdg',
                                                 '/etc/xdg', '/etc'])


class OSXTestCases(FakeSystem):
    SYS_NAME = 'Darwin'

    def test_mac_dirs(self):
        self.assertEqual(confuse.config_dirs(),
                         ['/Users/test/.config',
                          '/Users/test/Library/Application Support',
                          '/etc/xdg', '/etc'])

    def test_xdg_config_dirs(self):
        os.environ['XDG_CONFIG_DIRS'] = '/usr/local/etc/xdg:/etc/xdg'
        self.assertEqual(confuse.config_dirs(),
                         ['/Users/test/.config',
                          '/Users/test/Library/Application Support',
                          '/usr/local/etc/xdg',
                          '/etc/xdg', '/etc'])


class WindowsTestCases(FakeSystem):
    SYS_NAME = 'Windows'

    def test_dir_from_environ(self):
        self.assertEqual(confuse.config_dirs(),
                         ['C:\\Users\\test\\AppData\\Roaming',
                          'C:\\Users\\test\\winconfig'])

    def test_fallback_dir(self):
        del os.environ['APPDATA']
        self.assertEqual(confuse.config_dirs(),
                         ['C:\\Users\\test\\AppData\\Roaming'])


class ConfigFilenamesTest(unittest.TestCase):
    def setUp(self):
        self._old = os.path.isfile, confuse.yaml_util.load_yaml
        os.path.isfile = lambda x: True
        confuse.yaml_util.load_yaml = lambda *args, **kwargs: {}

    def tearDown(self):
        confuse.yaml_util.load_yaml, os.path.isfile = self._old

    def test_no_sources_when_files_missing(self):
        config = confuse.Configuration('myapp', read=False)
        filenames = [s.filename for s in config.sources]
        self.assertEqual(filenames, [])

    def test_search_package(self):
        config = confuse.Configuration('myapp', __name__, read=False)
        config._add_default_source()

        for source in config.sources:
            if source.default:
                default_source = source
                break
        else:
            self.fail("no default source")

        self.assertEqual(
            default_source.filename,
            os.path.join(os.path.dirname(__file__), 'config_default.yaml')
        )
        self.assertTrue(source.default)


class EnvVarTest(FakeSystem):
    TMP_HOME = True

    def setUp(self):
        super(EnvVarTest, self).setUp()
        self.config = confuse.Configuration('myapp', read=False)
        os.environ['MYAPPDIR'] = self.home  # use the tmp home as a config dir

    def test_env_var_name(self):
        self.assertEqual(self.config._env_var, 'MYAPPDIR')

    def test_env_var_dir_has_first_priority(self):
        self.assertEqual(self.config.config_dir(), self.home)

    def test_env_var_missing(self):
        del os.environ['MYAPPDIR']
        self.assertNotEqual(self.config.config_dir(), self.home)


class PrimaryConfigDirTest(FakeSystem):
    SYS_NAME = 'Linux'  # conversion from posix to nt is easy
    TMP_HOME = True

    if platform.system() == 'Windows':
        # wrap these functions as they need to work on the host system which is
        # only needed on Windows as we are using `posixpath`
        def join(self, *args):
            return self.os_path.normpath(self.os_path.join(*args))

        def makedirs(self, path, *args):
            os.path, os_path = self.os_path, os.path
            self._makedirs(path, *args)
            os.path = os_path

    def setUp(self):
        super(PrimaryConfigDirTest, self).setUp()
        if hasattr(self, 'join'):
            os.path.join = self.join
            os.makedirs, self._makedirs = self.makedirs, os.makedirs

        self.config = confuse.Configuration('test', read=False)

    def tearDown(self):
        super(PrimaryConfigDirTest, self).tearDown()
        if hasattr(self, '_makedirs'):
            os.makedirs = self._makedirs

    def test_create_dir_if_none_exists(self):
        path = os.path.join(self.home, '.config', 'test')
        assert not os.path.exists(path)

        self.assertEqual(self.config.config_dir(), path)
        self.assertTrue(os.path.isdir(path))

    def test_return_existing_dir(self):
        path = os.path.join(self.home, 'xdgconfig', 'test')
        os.makedirs(path)
        _touch(os.path.join(path, confuse.CONFIG_FILENAME))
        self.assertEqual(self.config.config_dir(), path)

    def test_do_not_create_dir_if_lower_priority_exists(self):
        path1 = os.path.join(self.home, 'xdgconfig', 'test')
        path2 = os.path.join(self.home, '.config', 'test')
        os.makedirs(path2)
        _touch(os.path.join(path2, confuse.CONFIG_FILENAME))
        assert not os.path.exists(path1)
        assert os.path.exists(path2)

        self.assertEqual(self.config.config_dir(), path2)
        self.assertFalse(os.path.isdir(path1))
        self.assertTrue(os.path.isdir(path2))



"""Painless YAML configuration.
"""

from __future__ import division, absolute_import, print_function

__version__ = '1.7.0'

from .exceptions import * # NOQA
from .util import * # NOQA
from .yaml_util import * # NOQA
from .sources import * # NOQA
from .templates import * # NOQA
from .core import * # NOQA



"""An example application using Confuse for configuration."""
from __future__ import division, absolute_import, print_function
import confuse
import argparse


template = {
    'library': confuse.Filename(),
    'import_write': confuse.OneOf([bool, 'ask', 'skip']),
    'ignore': confuse.StrSeq(),
    'plugins': list,

    'paths': {
        'directory': confuse.Filename(),
        'default': confuse.Filename(relative_to='directory'),
    },

    'servers': confuse.Sequence(
        {
            'hostname': str,
            'options': confuse.StrSeq(),
        }
    )
}

config = confuse.LazyConfig('ConfuseExample', __name__)


def main():
    parser = argparse.ArgumentParser(description='example Confuse program')
    parser.add_argument('--library', '-l', dest='library', metavar='LIBPATH',
                        help='library database file')
    parser.add_argument('--directory', '-d', dest='paths.directory',
                        metavar='DIRECTORY',
                        help='destination music directory')
    parser.add_argument('--verbose', '-v', dest='verbose', action='store_true',
                        help='print debugging messages')

    args = parser.parse_args()
    config.set_args(args, dots=True)

    print('configuration directory is', config.config_dir())

    # Use a boolean flag and the transient overlay.
    if config['verbose']:
        print('verbose mode')
        config['log']['level'] = 2
    else:
        config['log']['level'] = 0
    print('logging level is', config['log']['level'].get(int))

    valid = config.get(template)

    # Some validated/converted values.
    print('library is', valid.library)
    print('directory is', valid.paths.directory)
    print('paths.default is', valid.paths.default)
    print('servers are', [s.hostname for s in valid.servers])



from __future__ import division, absolute_import, print_function

try:
    import enum
    SUPPORTS_ENUM = True
except ImportError:
    SUPPORTS_ENUM = False

import confuse
import os
import unittest
from . import _root


class TypeCheckTest(unittest.TestCase):
    def test_str_type_correct(self):
        config = _root({'foo': 'bar'})
        value = config['foo'].get(str)
        self.assertEqual(value, 'bar')

    def test_str_type_incorrect(self):
        config = _root({'foo': 2})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(str)

    def test_int_type_correct(self):
        config = _root({'foo': 2})
        value = config['foo'].get(int)
        self.assertEqual(value, 2)

    def test_int_type_incorrect(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].get(int)


class BuiltInValidatorTest(unittest.TestCase):
    def test_as_filename_with_non_file_source(self):
        config = _root({'foo': 'foo/bar'})
        value = config['foo'].as_filename()
        self.assertEqual(value, os.path.join(os.getcwd(), 'foo', 'bar'))

    def test_as_filename_with_file_source(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml')
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        value = config['foo'].as_filename()
        self.assertEqual(value, os.path.realpath('/config/path/foo/bar'))

    def test_as_filename_with_default_source(self):
        source = confuse.ConfigSource({'foo': 'foo/bar'},
                                      filename='/baz/config.yaml',
                                      default=True)
        config = _root(source)
        config.config_dir = lambda: '/config/path'
        value = config['foo'].as_filename()
        self.assertEqual(value, os.path.realpath('/config/path/foo/bar'))

    def test_as_filename_wrong_type(self):
        config = _root({'foo': None})
        with self.assertRaises(confuse.ConfigTypeError):
            config['foo'].as_filename()

    def test_as_path(self):
        config = _root({'foo': 'foo/bar'})
        path = os.path.join(os.getcwd(), 'foo', 'bar')
        try:
            import pathlib
        except ImportError:
            with self.assertRaises(ImportError):
                value = config['foo'].as_path()
        else:
            value = config['foo'].as_path()
            path = pathlib.Path(path)
            self.assertEqual(value, path)

    def test_as_choice_correct(self):
        config = _root({'foo': 'bar'})
        value = config['foo'].as_choice(['foo', 'bar', 'baz'])
        self.assertEqual(value, 'bar')

    def test_as_choice_error(self):
        config = _root({'foo': 'bar'})
        with self.assertRaises(confuse.ConfigValueError):
            config['foo'].as_choice(['foo', 'baz'])

    def test_as_choice_with_dict(self):
        config = _root({'foo': 'bar'})
        res = config['foo'].as_choice({
            'bar': 'baz',
            'x': 'y',
        })
        self.assertEqual(res, 'baz')

    @unittest.skipUnless(SUPPORTS_ENUM,
                         "enum not supported in this version of Python.")
    def test_as_choice_with_enum(self):
        class Foobar(enum.Enum):
            Foo = 'bar'

        config = _root({'foo': Foobar.Foo.value})
        res = config['foo'].as_choice(Foobar)
        self.assertEqual(res, Foobar.Foo)

    @unittest.skipUnless(SUPPORTS_ENUM,
                         "enum not supported in this version of Python.")
    def test_as_choice_with_enum_error(self):
        class Foobar(enum.Enum):
            Foo = 'bar'

        config = _root({'foo': 'foo'})
        with self.assertRaises(confuse.ConfigValueError):
            config['foo'].as_choice(Foobar)

    def test_as_number_float(self):
        config = _root({'f': 1.0})
        config['f'].as_number()

    def test_as_number_int(self):
        config = _root({'i': 2})
        config['i'].as_number()

    @unittest.skipIf(confuse.PY3, "long only present in Python 2")
    def test_as_number_long_in_py2(self):
        config = _root({'l': long(3)})  # noqa ignore=F821
        config['l'].as_number()

    def test_as_number_string(self):
        config = _root({'s': 'a'})
        with self.assertRaises(confuse.ConfigTypeError):
            config['s'].as_number()

    def test_as_str_seq_str(self):
        config = _root({'k': 'a b c'})
        self.assertEqual(
            config['k'].as_str_seq(),
            ['a', 'b', 'c']
        )

    def test_as_str_seq_list(self):
        config = _root({'k': ['a b', 'c']})
        self.assertEqual(
            config['k'].as_str_seq(),
            ['a b', 'c']
        )

    def test_as_str(self):
        config = _root({'s': 'foo'})
        config['s'].as_str()

    def test_as_str_non_string(self):
        config = _root({'f': 1.0})
        with self.assertRaises(confuse.ConfigTypeError):
            config['f'].as_str()

    def test_as_str_expanded(self):
        config = _root({'s': '${CONFUSE_TEST_VAR}/bar'})
        os.environ["CONFUSE_TEST_VAR"] = 'foo'
        self.assertEqual(config['s'].as_str_expanded(), 'foo/bar')

    def test_as_pairs(self):
        config = _root({'k': [{'a': 'A'}, 'b', ['c', 'C']]})
        self.assertEqual(
            [('a', 'A'), ('b', None), ('c', 'C')],
            config['k'].as_pairs()
        )



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['confuse']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml']

extras_require = \
{"test:python_version == '2.7'": ['pathlib']}

setup(name='confuse',
      version='1.7.0',
      description='Painless YAML configuration.',
      author='Adrian Sampson',
      author_email='adrian@radbox.org',
      url='https://github.com/beetbox/confuse',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4',
     )



from __future__ import division, absolute_import, print_function

import confuse
import yaml
import unittest
from . import TempDir


def load(s):
    return yaml.load(s, Loader=confuse.Loader)


class ParseTest(unittest.TestCase):
    def test_dict_parsed_as_ordereddict(self):
        v = load("a: b\nc: d")
        self.assertTrue(isinstance(v, confuse.OrderedDict))
        self.assertEqual(list(v), ['a', 'c'])

    def test_string_beginning_with_percent(self):
        v = load("foo: %bar")
        self.assertEqual(v['foo'], '%bar')


class FileParseTest(unittest.TestCase):
    def _parse_contents(self, contents):
        with TempDir() as temp:
            path = temp.sub('test_config.yaml', contents)
            return confuse.load_yaml(path)

    def test_load_file(self):
        v = self._parse_contents(b'foo: bar')
        self.assertEqual(v['foo'], 'bar')

    def test_syntax_error(self):
        try:
            self._parse_contents(b':')
        except confuse.ConfigError as exc:
            self.assertTrue('test_config.yaml' in exc.name)
        else:
            self.fail('ConfigError not raised')

    def test_reload_conf(self):
        with TempDir() as temp:
            path = temp.sub('test_config.yaml', b'foo: bar')
            config = confuse.Configuration('test', __name__)
            config.set_file(filename=path)
            self.assertEqual(config['foo'].get(), 'bar')
            temp.sub('test_config.yaml', b'foo: bar2\ntest: hello world')
            config.reload()
            self.assertEqual(config['foo'].get(), 'bar2')
            self.assertEqual(config['test'].get(), 'hello world')

    def test_tab_indentation_error(self):
        try:
            self._parse_contents(b"foo:\n\tbar: baz")
        except confuse.ConfigError as exc:
            self.assertTrue('found tab' in exc.args[0])
        else:
            self.fail('ConfigError not raised')


class StringParseTest(unittest.TestCase):
    def test_load_string(self):
        v = confuse.load_yaml_string('foo: bar', 'test')
        self.assertEqual(v['foo'], 'bar')

    def test_string_syntax_error(self):
        try:
            confuse.load_yaml_string(':', 'test')
        except confuse.ConfigError as exc:
            self.assertTrue('test' in exc.name)
        else:
            self.fail('ConfigError not raised')

    def test_string_tab_indentation_error(self):
        try:
            confuse.load_yaml_string('foo:\n\tbar: baz', 'test')
        except confuse.ConfigError as exc:
            self.assertTrue('found tab' in exc.args[0])
        else:
            self.fail('ConfigError not raised')


class ParseAsScalarTest(unittest.TestCase):
    def test_text_string(self):
        v = confuse.yaml_util.parse_as_scalar('foo', confuse.Loader)
        self.assertEqual(v, 'foo')

    def test_number_string_to_int(self):
        v = confuse.yaml_util.parse_as_scalar('1', confuse.Loader)
        self.assertIsInstance(v, int)
        self.assertEqual(v, 1)

    def test_number_string_to_float(self):
        v = confuse.yaml_util.parse_as_scalar('1.0', confuse.Loader)
        self.assertIsInstance(v, float)
        self.assertEqual(v, 1.0)

    def test_bool_string_to_bool(self):
        v = confuse.yaml_util.parse_as_scalar('true', confuse.Loader)
        self.assertIs(v, True)

    def test_empty_string_to_none(self):
        v = confuse.yaml_util.parse_as_scalar('', confuse.Loader)
        self.assertIs(v, None)

    def test_null_string_to_none(self):
        v = confuse.yaml_util.parse_as_scalar('null', confuse.Loader)
        self.assertIs(v, None)

    def test_dict_string_unchanged(self):
        v = confuse.yaml_util.parse_as_scalar('{"foo": "bar"}', confuse.Loader)
        self.assertEqual(v, '{"foo": "bar"}')

    def test_dict_unchanged(self):
        v = confuse.yaml_util.parse_as_scalar({'foo': 'bar'}, confuse.Loader)
        self.assertEqual(v, {'foo': 'bar'})

    def test_list_string_unchanged(self):
        v = confuse.yaml_util.parse_as_scalar('["foo", "bar"]', confuse.Loader)
        self.assertEqual(v, '["foo", "bar"]')

    def test_list_unchanged(self):
        v = confuse.yaml_util.parse_as_scalar(['foo', 'bar'], confuse.Loader)
        self.assertEqual(v, ['foo', 'bar'])

    def test_invalid_yaml_string_unchanged(self):
        v = confuse.yaml_util.parse_as_scalar('!', confuse.Loader)
        self.assertEqual(v, '!')
